
; *
; * GB Printer Emulator V1.0
; *  by Jeff Frohwein
; *
; * Compiled with RGBASM.
; *

VIEWPIC equ     1

RGBSet:         MACRO
        DW      ((\3 >> 3) << 10) + ((\2 >> 3) << 5) + (\1 >> 3)
        ENDM

        INCLUDE "hardware.inc"
        INCLUDE "load1.inc"
        INCLUDE "ibmpc1.inc"

        SECTION "High Ram",HRAM

SerSend:        DB

        SECTION "Low Ram",BSS

PacketLen:      DW
Test:           DB
SerState:       DB
SerPtr:         DW

SerBuf:         DS      4096


        SECTION "Org $0",HOME
        ret

        SECTION "LCDC Int",HOME[$48]
        jp      LCDCInt

        SECTION "Serial Int",HOME[$58]
        push    af
        push    bc

        ld      a,[rSB]
        ld      b,a

        ld      a,[SerSend]
        ld      [rSB],a

        ld      a,$80
        ld      [rSC],a

        push    hl

        ld      a,[SerState]
        add     a
        add     a
        ld      l,a
        ld      h,2
        jp      [hl]

; Toggle character sets depending on position on screen

LCDCInt:
        push    af
.loop:  ld      a,[rSTAT]
        and     2
        jr      nz,.loop
        ld      a,[rLCDC]
        xor     $10
        ld      [rLCDC],a

        ld      a,[rLYC]
        xor     $c0
        ld      [rLYC],a
        pop     af
        reti

        SECTION "Org $100",HOME[$100]

;*** Beginning of rom execution point ***

        nop
        jp      Start

        NINTENDO_LOGO                   ; Nintendo graphic logo

;Rom Header Info

;    0123456789ABCDEF
 DB "GB PRTR EMU V1.0",0      ; Cart name   16bytes
 DB 0,0,0                     ; Not used
 DB CART_ROM_MBC1                  ; Cart type
 DB CART_ROM_256K             ; ROM Size (in bits)
 DB CART_RAM_NONE             ; RAM Size (in bits)
 DB 1,$33                     ; Maker ID
 DB 0                         ; Version     =0
 DB 0                         ; Complement check (important)
 DW 0                         ; Checksum (not important)

        SECTION "Org $200",HOME[$200]

; $88
        jp      SerialDoneInc
        nop
; $33
        jp      SerialDoneInc
        nop
; Command type
        jp      StoreCommand
        nop
; $00
        jp      SerialDoneInc
        nop
; Length byte (low)
        jp      SaveLowLength
        nop
; Length byte (high)
        jp      SaveHighLength
        nop
; Packet Data
        jp      SaveData
        nop
; CRC Low
        jp      SerialAck
        nop
; CRC High
        jp      SerialNAck
        nop
; Command 00
        jp      SerialDoneInc
        nop
; Command 01
        jp      SerialReset

SerialReset:
        ld      hl,SerState
        ld      [hl],0
        jr      SerialDone

SerialDoneInc:
        ld      hl,SerState
SerialDoneInc2:
        inc     [hl]

SerialDone:
;        xor     a
SerialDone2:
;        ld      [rSB],a

;        ld      a,$80
;        ld      [rSC],a

        pop     hl
        pop     bc
        pop     af
        reti

SerialAck:
        ld      hl,SerState
        inc     [hl]

        ld      a,$81
        ld      [SerSend],a
        jr      SerialDone2

SerialNAck:
        ld      hl,SerState
        inc     [hl]

        xor     a
        ld      [SerSend],a
        jr      SerialDone2

StoreCommand:
 if VIEWPIC
 else
        ld16r   hl,SerPtr

        ld      a,b
        ld      [hl+],a

        ld16r   SerPtr,hl
 endc
        jr      SerialDoneInc

SaveLowLength:

        ld      a,b
        ld      [PacketLen],a
        jr      SerialDoneInc

SaveHighLength:
        ld      a,b
        ld      [PacketLen+1],a
        ld      h,a
        ld      a,[PacketLen]
        or      h                       ; Is there any data?
        jr      nz,SerialDoneInc        ; yes

        ld      hl,SerState             ; no skip data collect state
        inc     [hl]
        jr      SerialDoneInc2

SaveData:
 if VIEWPIC
        ld16r   hl,SerPtr

        ld      a,b
        ld      [hl+],a

        ld16r   SerPtr,hl
 endc
        ld16r   hl,PacketLen
        dec     hl
        ld16r   PacketLen,hl

        ld      a,h
        or      l
        jr      nz,SerialDone

        jr      SerialDoneInc

        INCLUDE "memory1.asm"   ; memory set/copy

Start:
        di
        ld      sp,$ffff

;        call    Toggle

;        ld      hl,DefaultCGBPalettes
;        call    SetCGB_BGP

        xor     a
        ld      [SerState],a
        ld      [SerSend],a
        ld      [rSB],a

        ld      a,$80
        ld      [rSC],a

        ld      a,$40
        ld      [rSTAT],a

        ld      a,80
        ld      [rLYC],a

        ld      a,8
        ld      [rIE],a

        ld      hl,SerBuf
        ld16r   SerPtr,hl

        ei

        ld      hl,SerBuf
        ld      bc,100
        ld      a,$69
        call    mem_Set

        call    DisplayPicture        ; Display picture on screen
        call    WaitForBytesThenDisplay
LoopForever:
        jr      LoopForever

WaitForBytesThenDisplay:
.wait:
        ld      a,[rSC]
        bit     7,a
        jr      nz,.wait

        call    Delay

 if VIEWPIC
        call    DisplayOff

        ld      a,8+2
        ld      [rIE],a

        ld      hl,SerBuf
        ld      de,$8000
        ld      bc,360*16
        call    mem_Copy

        call    SetAPATileMap1

        ld      hl,rLCDC
        set     7,[hl]          ; Turn on display
 else

        ld      de,SerBuf
        ld      hl,$9840

        ld      c,32
.loop
        ld      a,[de]
        inc     de
        call    DisplayHexByte

        dec     c
        jr      nz,.loop
 endc
        ret

Delay:
        ld      b,16
.loop2:
        ld      hl,0
.loop:  dec     hl
        ld      a,h
        or      l
        jr      nz,.loop

        dec     b
        jr      nz,.loop2

        ret

DisplayHexByte:
        swap    a
        call    DisplayNibble
        swap    a
        call    DisplayNibble
        inc     hl
        ret

DisplayNibble:
        push    af
        and     $f
        cp      9+1
        jr      c,.small
        add     "a"-"0"-10
.small:
        add     "0"
        ld      b,a

        di
        lcd_WaitVRAM
        ld      a,b
        ld      [hl+],a
        ei

        pop     af
        ret

; * Toggle GBC single / double speed mode *

Toggle:
        di

        ld      hl,rIE
        ld      a,[hl]
        push    af

        xor     a
        ld      [hl],a         ;disable interrupts
        ld      [rIF],a

        ld      a,$30
        ld      [rP1],a

        ld      a,1
        ld      [rKEY1],a

        stop

        pop     af
        ld      [hl],a

        ei
        ret

; * Display a picture *

DisplayPicture:
        call    DisplayOff

        ld      hl,PicTileSet
        ld      de,$8000
        ld      bc,$1000
        call    mem_CopyMono        ; Copy tile set to memory

        ld      hl,PicMap
        ld      de,$9800
        call    Copy20x18       ; Copy tile map to memory

        ld      a,$e4
        ld      [rBGP],a        ; Setup default background colors

        xor     a
        ld      [rSCX],a
        ld      [rSCY],a

   ;     call    DisplayHexByte

        ld      a,LCDCF_ON|LCDCF_BG8000|LCDCF_BG9800|LCDCF_BGON|LCDCF_OBJ16|LCDCF_OBJOFF
        ld      [rLCDC],a         ; Turn screen on

        nop
        nop
        nop

        ret

; Copy char map to screen
; Entry: HL = Source Map
;        DE = Destination Screen

Copy20x18:
        ld      b,18
Copy20xB:
        ld      c,20
Copy202:
        ld      a,[hl+]
        ld      [de],a
        inc     de
        dec     c
        jr      nz,Copy202

        push    hl
        ld      hl,12
        add     hl,de
        ld16    de,hl
        pop     hl

        dec     b
        jr      nz,Copy20xB

        ret

; *** Fill screen with APA tiles ***

SetAPATileMap1:
        ld      hl,_SCRN0
        jr      _initscrn

SetAPATileMap2:
        ld      hl,_SCRN1
_initscrn:
        xor     a

        ld      bc,12
        ld      e,18
.loop1:
        ld      d,20
.loop2:
        ld      [hl+],a
        inc     a
        dec     d
        jr      nz,.loop2
        add     hl,bc
        dec     e
        jr      nz,.loop1
        ret

; *** Turn off LCD Display ***

ScreenOff:
        ld      hl,rLCDC
        bit     7,[hl]          ; Is LCD already off?
        ret     z               ; yes, exit

        ld      a,[rIE]
        push    af
        res     0,a
        ld      [rIE],a         ; Disable vblank interrupt if enabled

.loop:  ld      a,[rLY]         ; Loop until in first part of vblank
        cp      145
        jr      nz,.loop

        res     7,[hl]          ; Turn the screen off

        pop     af
        ld      [rIE],a         ; Restore the state of vblank interrupt
        ret

; *** Set background palettes ***
; Entry: HL = Pointer to BG palettes

SetCGB_BGP:
        ld      a,$80
        ld      [rBCPS],a

        ld      bc,$4069        ; b = 64, c = $69
.loop:
        ld      a,[hl+]
        ld      [c],a
        dec     b
        jr      nz,.loop

        ret

DefaultCGBPalettes:
        RGBSet  255,255,255     ; White
        RGBSet  191,191,191     ; Grey
        RGBSet  127,127,127     ; Dark
        RGBSet  0,0,0           ; Black

PicTileSet:
        chr_IBMPC1 1,8

PicMap:
;                12345678901234567890
        db      "  Ŀ  "
        db      "    GB Printer    "
        db      "   Emulator v1.0  "
        db      "    "
        db      "  by Jeff Frohwein  "
        db      "                    "
        db      "  Connect another   "
        db      " GB to the link port"
        db      " and start printing "
        db      " on the other unit. "
        db      "                    "
        db      "  This Screen will  "
        db      " display the picture"
        db      " once picture xfer  "
        db      " is complete.       "
        db      "                    "
        db      "                    "
        db      "                    "
